
namespace TMXMappacks {
    // tmx short link of s/m/ or of mappack/ but without map name on the end
    bool LooksLikeShortLink(const string &in searchname) {
        if (Regex::IsMatch(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]s[\/\\]m[\/\\]\d{1,10}$""")) {
            return true;
        }
        return false;
    }
    // bool LooksLikeShortLink2(const string &in searchname) {
    //     if (Regex::IsMatch(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]mappack[\/\\]\d{1,10}$""")) {
    //         return true;
    //     }
    //     return false;
    // }
    bool LooksLikeFullLink(const string &in searchname) {
        if (Regex::Contains(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]mappack[\/\\]\d{1,10}""")) {
            return true;
        }
        return false;
    }
    bool LooksLikeFullLink2(const string &in searchname) {
        if (Regex::Contains(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]mappack[\/\\]view[\/\\]\d{1,10}""")) {
            return true;
        }
        return false;
    }

    void SetMapsFromNumber(const string &in mappacknumber, const string &in name) {
        SetMaps('https://trackmania.exchange/api/mappack/get_mappack_tracks/' + mappacknumber, name, mappacknumber);
    }

    void SetMapsFromNumber(const string &in mappacknumber) {
        SetMaps('https://trackmania.exchange/api/mappack/get_mappack_tracks/' + mappacknumber, mappacknumber, mappacknumber);
    }

    void SetMapsFromShortLink(const string &in mappacklink) {
        if (mappacklink.StartsWith('https:')) { // not checking for \\ or // because it can be either
            SetMapsFromNumber(mappacklink.SubStr(32));
        } else if (mappacklink.StartsWith('http:')) {
            SetMapsFromNumber(mappacklink.SubStr(31));
        } else {
            SetMapsFromNumber(mappacklink.SubStr(24));
        }
    }
    void SetMapsFromFullLink(string &in mappacklink) {
        if (mappacklink.StartsWith('https:')) {
            mappacklink = mappacklink.SubStr(36);
        } else if (mappacklink.StartsWith('http:')) {
            mappacklink = mappacklink.SubStr(35);
        } else {
            mappacklink = mappacklink.SubStr(28);
        }
        // print(mappacklink);
        string name;
        if (mappacklink.Contains("/")) {
            if (mappacklink.IndexOf("/") < mappacklink.Length - 1) {
                name = mappacklink.SubStr(mappacklink.IndexOf("/") + 1);
            }
            SetMapsFromNumber(mappacklink.SubStr(0, mappacklink.IndexOf("/")), name);
        } else if (mappacklink.Contains("\\")) {
            if (mappacklink.IndexOf("\\") < mappacklink.Length - 1) {
                name = mappacklink.SubStr(mappacklink.IndexOf("\\") + 1);
            }
            SetMapsFromNumber(mappacklink.SubStr(0, mappacklink.IndexOf("\\")), name);
        } else if (mappacklink.Length > 0) {
            SetMapsFromNumber(mappacklink);
        }
    }
    void SetMapsFromFullLink2(string &in mappacklink) {
        if (mappacklink.StartsWith('https:')) {
            mappacklink = mappacklink.SubStr(41);
        } else if (mappacklink.StartsWith('http:')) {
            mappacklink = mappacklink.SubStr(40);
        } else {
            mappacklink = mappacklink.SubStr(33);
        }
        // print(mappacklink);
        string name;
        if (mappacklink.Contains("/")) {
            if (mappacklink.IndexOf("/") < mappacklink.Length - 1) {
                name = mappacklink.SubStr(mappacklink.IndexOf("/") + 1);
            }
            SetMapsFromNumber(mappacklink.SubStr(0, mappacklink.IndexOf("/")), name);
        } else if (mappacklink.Contains("\\")) {
            if (mappacklink.IndexOf("\\") < mappacklink.Length - 1) {
                name = mappacklink.SubStr(mappacklink.IndexOf("\\") + 1);
            }
            SetMapsFromNumber(mappacklink.SubStr(0, mappacklink.IndexOf("\\")), name);
        } else if (mappacklink.Length > 0) {
            SetMapsFromNumber(mappacklink);
        }
    }

    void SetMaps(const string &in infourl, const string &in packname, const string &in packid) {
        for (int i = maplists.Length - 1; i >= 0; i--) {
            if (maplists[i].identifier == "tmx:"+packid) {
                return;
                // maplists.RemoveAt(i);
            }
        }
        Net::HttpRequest@ req = Net::HttpGet(infourl);
        req.Start();
        while(!req.Finished()) {yield();}
        if (req.ResponseCode() != 200 || req.String().Length == 0) {
            print('invalid http response when getting tmx mappack' + req.ResponseCode() + ', ' + req.String());
            return;
        }
        Json::Value@ response = Json::Parse(req.String());
        if (response.GetType() != Json::Type::Array) {
            print('invalid response type when getting tmx mappack' + response.GetType() + ', ' + req.String());
            return;
        }
        MapList@ newmaps = MapList(packname, "tmx:"+packid, MapListType::TMXMappack, false);
        for (uint i = 0; i < response.Length; i++) {
            newmaps.InsertLast(Map(Text::OpenplanetFormatCodes(response[i]["GbxMapName"]), tostring(int(response[i]["TrackID"])), string(response[i]["MapType"]), 'https://trackmania.exchange/maps/download/' + int(response[i]["TrackID"])));
        }
        maplists.InsertFirst(newmaps);

    }
}


/*
https://trackmania.exchange/mappack/2614/magnet-altered-nadeo-campaigns
https://trackmania.exchange/mappack/2614
https://trackmania.exchange/mappack/view/2614/magnet-altered-nadeo-campaigns
https://trackmania.exchange/mappack/view/2614
https://trackmania.exchange/s/m/2614
http://trackmania.exchange/mappack/2614/magnet-altered-nadeo-campaigns
http://trackmania.exchange/mappack/2614
http://trackmania.exchange/mappack/view/2614/magnet-altered-nadeo-campaigns
http://trackmania.exchange/mappack/view/2614
http://trackmania.exchange/s/m/2614
trackmania.exchange/mappack/2614/magnet-altered-nadeo-campaigns
trackmania.exchange/mappack/2614
trackmania.exchange/mappack/view/2614/magnet-altered-nadeo-campaigns
trackmania.exchange/mappack/view/2614
trackmania.exchange/s/m/2614
*/