
namespace TMXMaps {
    bool LooksLikeNumber(const string &in searchname) {
        if (Regex::IsMatch(searchname, """\d{1,10}""")) {
            return true;
        }
        return false;
    }
    // tmx short link of s/tr/ or of maps/ but without map name on the end
    bool LooksLikeShortLink(const string &in searchname) {
        if (Regex::IsMatch(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\](s[\/\\]tr|maps)[\/\\]\d{1,10}$""")) {
            return true;
        }
        return false;
    }
    bool LooksLikeFullLink(const string &in searchname) {
        if (Regex::Contains(searchname, """^(https?:[\/\\]{2})?trackmania\.exchange[\/\\]maps[\/\\]\d{1,10}[\/\\]""")) {
            return true;
        }
        return false;
    }

    string GetDownloadFromNumber(const string &in mapnumber, string &out type, string &out mapname) {
        Net::HttpRequest@ req = Net::HttpRequest();
        req.Method = Net::HttpMethod::Get;
        req.Url = 'https://trackmania.exchange/api/maps/get_map_info/id/' + mapnumber;
        req.Start();
        while (!req.Finished()) {
            yield();
        }
        if (req.String().Length == 0) {
            print('map name get');
            UI::ShowNotification("Play map extended", "Couldn't find map name");
            type = "";
            mapname = mapnumber;
        } else {
            Json::Value response = Json::Parse(req.String());
            type = response["MapType"];
            mapname = response["GbxMapName"];
        }
        return 'https://trackmania.exchange/maps/download/' + mapnumber;
    }

    string GetDownloadFromShortLink(const string &in maplink, string &out type, string &out mapname) {
        return GetDownloadFromNumber(GetNumberFromShortLink(maplink), type, mapname);
    }
    string GetNumberFromShortLink(const string &in maplink) {
        if (maplink.StartsWith('https:')) {
            return maplink.SubStr(33);
        } else if (maplink.StartsWith('http:')) {
            return maplink.SubStr(32);
        } else {
            return maplink.SubStr(25);
        }
    }
    string GetDownloadFromFullLink(const string &in maplink, string &out type, string &out mapname) {
        return GetDownloadFromNumber(GetNumberFromFullLink(maplink), type, mapname);
    }
    string GetNumberFromFullLink(string &in maplink) {
        if (maplink.StartsWith('https:')) {
            maplink = maplink.SubStr(33);
        } else if (maplink.StartsWith('http:')) {
            maplink = maplink.SubStr(32);
        } else {
            maplink = maplink.SubStr(25);
        }
        if (maplink.Contains("/")) {
            return maplink.SubStr(0, maplink.IndexOf("/"));
        } else if (maplink.Contains("\\")) {
            return maplink.SubStr(0, maplink.IndexOf("\\"));
        }
        throw('cannot get number, invalid full link');
        return "";
    }

    string GetUidFromNumber(const string &in mapnumber) {
        Net::HttpRequest@ req = Net::HttpRequest();
        req.Method = Net::HttpMethod::Get;
        req.Url = 'https://trackmania.exchange/api/maps/get_map_info/id/' + mapnumber;
        req.Start();
        while (!req.Finished()) {
            yield();
        }
        if (req.String().Length == 0) {
            print('map name get');
            UI::ShowNotification("Play map extended", "Couldn't find map name");
            return "";
        } else {
            Json::Value response = Json::Parse(req.String());
            return response["TrackUID"];
        }
    }
    string GetUidFromShortLink(const string &in maplink) {
        if (maplink.StartsWith('https:')) {
            return GetUidFromNumber(maplink.SubStr(33));
        } else if (maplink.StartsWith('http:')) {
            return GetUidFromNumber(maplink.SubStr(32));
        } else {
            return GetUidFromNumber(maplink.SubStr(25));
        }
    }
    string GetUidFromFullLink(string &in maplink) {
        if (maplink.StartsWith('https:')) {
            maplink = maplink.SubStr(33);
        } else if (maplink.StartsWith('http:')) {
            maplink = maplink.SubStr(32);
        } else {
            maplink = maplink.SubStr(25);
        }
        if (maplink.Contains("/")) {
            return GetUidFromNumber(maplink.SubStr(0, maplink.IndexOf("/")));
        } else if (maplink.Contains("\\")) {
            return GetUidFromNumber(maplink.SubStr(0, maplink.IndexOf("\\")));
        }
        return "";
    }

    string GetIdFromUid(const string &in uid) {
        Net::HttpRequest@ req = Net::HttpRequest();
        req.Method = Net::HttpMethod::Get;
        req.Url = 'https://trackmania.exchange/api/maps/get_map_info/uid/' + uid;
        req.Start();
        while (!req.Finished()) {
            yield();
        }
        if (req.ResponseCode() != 200) {
            print('invalid response from tmx for id from uid');
            return "-1";
        }
        if (req.String().Length == 0) {
            // print('map not on tmx: ' + uid);
            return "-1";
        }
        Json::Value response = Json::Parse(req.String());
        return tostring(int(response['TrackID']));
        
    }
}
