enum LoadAs {
    Solo,
    Editor,
}

enum LooksType {
    None,
    Map,
    Activity,
}

string maptoload = "";
LoadAs loadmapas = LoadAs::Solo;

// from a XertroV plugin
void ReturnToMenu(bool yieldTillReady = false) {
    auto app = cast<CGameManiaPlanet>(GetApp());
    if (app.Network.PlaygroundClientScriptAPI.IsInGameMenuDisplayed) {
        app.Network.PlaygroundInterfaceScriptHandler.CloseInGameMenu(CGameScriptHandlerPlaygroundInterface::EInGameMenuResult::Quit);
    }
    app.BackToMainMenu();
    while (yieldTillReady && !app.ManiaTitleControlScriptAPI.IsReady) yield();
}
wstring GetUrl(const string &in identifier, string &out type, string &out mapname) {
    wstring url = "";
    if (NadeoMaps::LooksLikeUid(identifier)) {
        url = NadeoMaps::GetDownloadFromUid(identifier, type, mapname);
    } else if (TMXMaps::LooksLikeNumber(identifier)) {
        url = TMXMaps::GetDownloadFromNumber(identifier, type, mapname);
    } else if (TMXMaps::LooksLikeShortLink(identifier)) {
        url = TMXMaps::GetDownloadFromShortLink(identifier, type, mapname);
    } else if (TMXMaps::LooksLikeFullLink(identifier)) {
        url = TMXMaps::GetDownloadFromFullLink(identifier, type, mapname);
    } else if (TrackmaniaioMaps::LooksLikeMap(identifier)) {
        url = TrackmaniaioMaps::GetDownloadFromURL(identifier, type, mapname);
    // } else if (identifier.Contains(':\\') || identifier.Contains(':/')) {
    //     url = identifier;
    }
    return url;
}

void DisplayMaps(const string &in identifier) {
    if (TMXMappacks::LooksLikeShortLink(identifier)) {
        TMXMappacks::SetMapsFromShortLink(identifier);
    } else if (TMXMappacks::LooksLikeFullLink(identifier)) {
        TMXMappacks::SetMapsFromFullLink(identifier);
    } else if (TMXMappacks::LooksLikeFullLink2(identifier)) {
        TMXMappacks::SetMapsFromFullLink2(identifier);
    } else if (TrackmaniaioActivities::LooksLikeCampaign(identifier)) {
        TrackmaniaioActivities::SetMapsFromCampaign(identifier);
    } else if (TrackmaniaioActivities::LooksLikeRoom(identifier)) {
        TrackmaniaioActivities::SetMapsFromRoom(identifier);
    } else if (TrackmaniaioActivities::LooksLikeTrackUpload(identifier)) {
        TrackmaniaioActivities::SetMapsFromTrackUpload(identifier);
    } else if (TrackmaniaioActivities::LooksLikeSkinUpload(identifier)) {
        TrackmaniaioActivities::SetSkinsFromSkinUpload(identifier);
    } else if (TrackmaniaioActivities::LooksLikeClub(identifier)) {
        TrackmaniaioActivities::SetActivitiesFromClub(identifier);
    }
}

string GetUid(const string &in identifier) {
    string uid = "";
    if (NadeoMaps::LooksLikeUid(identifier)) {
        uid = identifier;
    } else if (TMXMaps::LooksLikeNumber(identifier)) {
        uid = TMXMaps::GetUidFromNumber(identifier);
    } else if (TMXMaps::LooksLikeShortLink(identifier)) {
        uid = TMXMaps::GetUidFromShortLink(identifier);
    } else if (TMXMaps::LooksLikeFullLink(identifier)) {
        uid = TMXMaps::GetUidFromFullLink(identifier);
    } else if (TrackmaniaioMaps::LooksLikeMap(identifier)) {
        uid = TrackmaniaioMaps::GetUidFromURL(identifier);
    }
    return uid;
}
string GetTMXId(const string &in identifier) {
    string id = "";
    if (NadeoMaps::LooksLikeUid(identifier)) {
        id = TMXMaps::GetIdFromUid(identifier);
    } else if (TMXMaps::LooksLikeNumber(identifier)) {
        id = identifier;
    } else if (TMXMaps::LooksLikeShortLink(identifier)) {
        id = TMXMaps::GetNumberFromShortLink(identifier);
    } else if (TMXMaps::LooksLikeFullLink(identifier)) {
        id = TMXMaps::GetNumberFromFullLink(identifier);
    } else if (TrackmaniaioMaps::LooksLikeMap(identifier)) {
        id = TMXMaps::GetIdFromUid(TrackmaniaioMaps::GetUidFromURL(identifier));
    }
    return id;
}

LooksType LooksLike(const string &in identifier) {
    if (identifier.Length == 0) {return LooksType::None;}
    if (identifier.EndsWith('/') || identifier.EndsWith('\\')) {
        return LooksType::None;
    }
    if (NadeoMaps::LooksLikeUid(identifier) ||
        TMXMaps::LooksLikeNumber(identifier) ||
        TMXMaps::LooksLikeShortLink(identifier) ||
        TMXMaps::LooksLikeFullLink(identifier) ||
        TrackmaniaioMaps::LooksLikeMap(identifier)
        ) {
        return LooksType::Map;
    }
    if (TMXMappacks::LooksLikeShortLink(identifier) ||
        TMXMappacks::LooksLikeFullLink(identifier) ||
        TMXMappacks::LooksLikeFullLink2(identifier) ||
        TrackmaniaioActivities::LooksLikeCampaign(identifier) ||
        TrackmaniaioActivities::LooksLikeRoom(identifier) ||
        TrackmaniaioActivities::LooksLikeTrackUpload(identifier) ||
        TrackmaniaioActivities::LooksLikeSkinUpload(identifier) ||
        TrackmaniaioActivities::LooksLikeClub(identifier)
        ) {
        return LooksType::Activity;
    }
    return LooksType::None;
}

bool checkPermissions(LoadAs loadmapas) {
    if (loadmapas == LoadAs::Solo) {
        if (Permissions::PlayLocalMap()) {
            return true;
        } else {
            UI::ShowNotification("Play map extended", "Playing maps is not available in your access tier");
            return false;
        }
    } else if (loadmapas == LoadAs::Editor) {
        if (Permissions::OpenAdvancedMapEditor()) {
            return true;
        } else {
            UI::ShowNotification("Play map extended", "Editing maps is not available in your access tier");
            return false;
        }
    } else {
        UI::ShowNotification("Play map extended", "Invalid load location (during permissions check)");
        return false;
    }
}
// called each frame to check if maptoload is not none 
// so can yield while loading a map
bool CheckMapToLoad() {
    if (maptoload == "") {
        return false;
    }
    if (!checkPermissions(loadmapas)) {
        maptoload = "";
        return false;
    }
    isLoadingMap = true;
    string type;
    string name;
    wstring url = GetUrl(maptoload, type, name);
    if (url == "") {
        print("invalid map");
        UI::ShowNotification("Play map extended", "Couldn't load map");
        maptoload = "";
        isLoadingMap = false;
        return true;
    }
    _PlayMapFromUrl(url, loadmapas, type, Text::OpenplanetFormatCodes(name));
    maptoload = "";
    isLoadingMap = false;
    return true;
}

void _PlayMapFromUrl(const string &in url, LoadAs loadas, const string &in type, const string &in name = "") {
    if (!checkPermissions(loadas)) {
        isLoadingMap = false;
        maptoload = "";
        return;
    }
    CGameManiaPlanet@ app = cast<CGameManiaPlanet>(GetApp());
    print('returning to menu');
    ReturnToMenu(true);

    string mode = type.ToLower();
    string settingsxml = "";
    if (mode.EndsWith("race")) {
        mode = "TrackMania\\TM_PlayMap_Local";
    } else if (mode.EndsWith("stunt")) {
        mode = "Trackmania\\TM_StuntSolo_Local";
    } else if (mode.EndsWith("platform")) {
        mode = "Trackmania\\TM_Platform_Local";
    } else if (mode.EndsWith("royal")) {
        mode = "Trackmania\\TM_RoyalTimeAttack_Local";
    } else {
        print('unknown mode: ' + mode);
        mode = "";
    }

    print('loading map: ' + name + ' with mode: ' + mode);

    while (!app.ManiaTitleControlScriptAPI.IsReady) {yield();}
    if (loadas == LoadAs::Solo) {
        app.ManiaTitleControlScriptAPI.PlayMap(url, mode, settingsxml);
    } else if (loadas == LoadAs::Editor) {
        app.ManiaTitleControlScriptAPI.EditMap(url, "", "");
    }
}

// called with startnew when download clicked
void SaveMap(const string &in identifier) {
    CGameManiaPlanet@ app = cast<CGameManiaPlanet>(GetApp());
    if (identifier == "") {
    }

    string type;
    string mapname;
    string url = GetUrl(identifier, type, mapname);
    if (url == "") {
        print("invalid map");
        UI::ShowNotification("Play map extended", "Couldn't download map");
        return;
    }
    _SaveMapFromUrl(url, Text::StripFormatCodes(mapname));
}

// if we already know the url we don't need to check what type of map it is
void _SaveMapFromUrl(const string &in url, const string &in filename) {
    Net::HttpRequest@ req = Net::HttpRequest();
    req.Method = Net::HttpMethod::Get;
    req.Url = url;
    req.Start();
    while (!req.Finished()) {
        yield();
    }
    if (req.String().Length == 0) {
        print('map download failed');
        UI::ShowNotification("Play map extended", "Couldn't download map");
        return;
    }
    // CheckDownloadsFolder() // will create a plugin folder in downloads if its not there already
    if (!IO::FolderExists(IO::FromUserGameFolder("Maps/Downloaded/PlayMapExtended"))) {
        IO::CreateFolder(IO::FromUserGameFolder("Maps/Downloaded/PlayMapExtended"), true);
    }
    req.SaveToFile(IO::FromUserGameFolder("Maps/Downloaded/PlayMapExtended/" + filename + '.Map.Gbx'));
    if (Draw::MeasureString("/Maps/Downloaded/PlayMapExtended/" + filename + ".Map.Gbx").x > 350 * UI::GetScale()) {
        UI::ShowNotification("Play map extended", "Saved map to /Maps/Downloaded/PlayMapExtended/\n" + filename + ".Map.Gbx");
    } else {
        UI::ShowNotification("Play map extended", "Saved map to /Maps/Downloaded/PlayMapExtended/" + filename + ".Map.Gbx");
    }
}

// called with startnew when favourite clicked
void FavouriteMap(const string &in identifier) {
    string uid = GetUid(identifier);
    _FavouriteMapFromUid(uid);
}
void _FavouriteMapFromUid(const string &in uid) {
    trace('favouriting: ' + uid);
    Favourites::isValid = false; Favourites::isRefreshing = true; // don't refresh until after added, but mark as invalid now
    Net::HttpRequest@ req = NadeoServices::Post("NadeoLiveServices", NadeoServices::BaseURLLive() + "/api/token/map/favorite/" + uid + "/add");
    req.Start();
    while(!req.Finished()) {yield();}
    if (req.ResponseCode() != 200) {
        print('map favouriting failed: error ' + req.ResponseCode());
        print(req.String());
    }
    // print(req.String());
    Favourites::Refresh();

}
void UnfavouriteMap(const string &in identifier) {
    string uid = GetUid(identifier);
    _UnfavouriteMapFromUid(uid);
}
void _UnfavouriteMapFromUid(const string &in uid) {
    Favourites::isValid = false; Favourites::isRefreshing = true; // don't refresh until after added, but mark as invalid now
    Net::HttpRequest@ req = NadeoServices::Post("NadeoLiveServices", NadeoServices::BaseURLLive() + "/api/token/map/favorite/" + uid + "/remove");
    req.Start();
    while(!req.Finished()) {yield();}
    if (req.ResponseCode() != 200) {
        print('map favouriting failed: error ' + req.ResponseCode());
    }
    // print(req.String());
    Favourites::Refresh();
}

    //https://live-services.trackmania.nadeo.live/api/token/map/favorite?offset=0&length=96&sort=date&order=desc&mapTypeList=TrackMania%5cTM_Race%2cTrackMania%5cTM_Stunt&playable=1
    //https://live-services.trackmania.nadeo.live/api/token/map/favorite?offset=0&length=96&sort=date&order=desc